/*
Example for Fram_2 Click

    Date          : Jul 2017.
    Author        : Djordje Rosic

Test configuration PIC :
    
    MCU             : P18F87K22
    Dev. Board      : EasyPIC PRO v7
    SW              : mikroC PRO for PIC v7.1.0

Description :

    This example demonstrates the use of FRAM 2 click. It will first initialize
    peripherals and enable writing to FRAM. It will then write the value 42
    into register 0x10, and then read that value and output it via UART.

    Note: Turn on the UART switches on the board to enable uart output.

*/


#include <stdint.h>

sbit FRAM2_CS         at LATE0_bit;
sbit FRAM2_CS_DIR     at TRISE0_bit;


static void Fram_2_Write (uint8_t address1, uint8_t address2,
                          uint8_t address3, uint8_t writeData);
static uint8_t Fram_2_Read (uint8_t address1,
                            uint8_t address2, uint8_t address3);

void systemInit()
{
    SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV16, _SPI_DATA_SAMPLE_MIDDLE,
                                              _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);

    UART1_Init ( 9600);
    FRAM2_CS_DIR = 0;
    UART_Write_Text ("\r\nInitialized");
}

void Fram_2_Init()
{
    FRAM2_CS = 0;
    SPI_Write (0x06); //Write enable command is required prior to any write.
    FRAM2_CS = 1;
}

void Fram_2_Task()
{
    uint8_t readData;
    char outputText [5];
    memset ( outputText, 0, 5);
    UART_Write_Text ("\r\nWriting value 42 into register 0x10...");
    Fram_2_Write (0x00, 0x00, 0x10, 42 );

    UART_Write_Text ("\r\nReading from register 0x10...");
    readData = Fram_2_Read  (0x00, 0x00, 0x10);
    ShortToStr (readData, outputText);
    UART_Write_Text ("\r\nRead value: ");
    UART_Write_Text (outputText);
}

void main()
{
    systemInit();
    Fram_2_Init();

    Fram_2_Task();
}

static void Fram_2_Write (uint8_t address1, uint8_t address2,
                          uint8_t address3, uint8_t writeData)
{
    FRAM2_CS = 0;
    SPI_Write (0x02);             //Write OPcode
    SPI_Write (address1);
    SPI_Write (address2);
    SPI_Write (address3);
    SPI_Write (writeData);
    //More data can be written after this. Address pointer is auto-incremented.
    FRAM2_CS = 1;
}
static uint8_t Fram_2_Read (uint8_t address1,
                            uint8_t address2, uint8_t address3)
{
    FRAM2_CS = 0;
    SPI_Write (0x03);             //Read OPcode
    SPI_Write (address1);
    SPI_Write (address2);
    SPI_Write (address3);
    return SPI_Read (0x00);
    //More data can be read after this. Address pointer is auto-incremented.
    FRAM2_CS = 1;
}